@@ -1,18 +1,49 @@
 package App::Wallflower;
-$App::Wallflower::VERSION = '1.005';
+$App::Wallflower::VERSION = '1.006';
 use strict;
 use warnings;
 
 use Getopt::Long qw( GetOptionsFromArray );
 use Pod::Usage;
+use Carp;
 use Plack::Util ();
 use URI;
 use Wallflower;
 use Wallflower::Util qw( links_from );
 
+sub _default_options {
+    return (
+        follow      => 1,
+        environment => 'deployment',
+        host        => ['localhost'],
+        verbose     => 1,
+        errors      => 1,
+    );
+}
+
+# [ activating option, coderef ]
+my @callbacks = (
+    [   errors => sub {
+            my ( $url, $response ) = @_;
+            my ( $status, $headers, $file ) = @$response;
+            return if $status == 200;
+            printf "$status %s%s\n", $url->path,
+                $file && " => $file [${\-s $file}]";
+            }
+    ], [verbose => sub {
+            my ( $url, $response ) = @_;
+            my ( $status, $headers, $file ) = @$response;
+            return if $status != 200;
+            printf "$status %s%s\n", $url->path,
+                $file && " => $file [${\-s $file}]";
+            }
+    ],
+);
+
 sub new_with_options {
     my ( $class, $args ) = @_;
     my $input = (caller)[1];
+    $args ||= [];
 
     # save previous configuration
     my $save = Getopt::Long::Configure();
@@ -21,17 +52,14 @@ sub new_with_options {
     Getopt::Long::ConfigDefaults();
 
     # get the command-line options (modifies $args)
-    my %option = (
-        follow      => 1,
-        environment => 'deployment',
-        host        => ['localhost']
-    );
+    my %option = _default_options();
     GetOptionsFromArray(
         $args,           \%option,
         'application=s', 'destination|directory=s',
         'index=s',       'environment=s',
         'follow!',       'filter|files|F',
         'quiet',         'include|INC=s@',
+        'verbose!',      'errors!',
         'host=s@',
         'url|uri=s',
         'help',          'manual',
@@ -64,10 +92,33 @@ sub new_with_options {
         -message => 'Missing required option: application'
     ) if !exists $option{application};
 
+    # create the object
+    return $class->new(
+        option => \%option,
+        args   => $args,
+    );
+
+}
+
+sub new {
+    my ( $class, %args ) = @_;
+    my %option = ( _default_options(), %{ $args{option} || {} } );
+    my $args   = $args{args} || [];
+    my @cb     = @{ $args{callbacks} || [] };
+
+    # application is required
+    croak "Option application is required" if !exists $option{application};
+
+    # --quiet = --no-verbose --no-errors
+    $option{verbose} = $option{errors} = 0 if $option{quiet};
+
     # add the hostname passed via --url to the list built with --host
     push @{ $option{host} }, URI->new( $option{url} )->host
        if $option{url};
 
+    # pre-defined callbacks
+    push @cb, map $_->[1], grep $option{ $_->[0] }, @callbacks;
+
     # include option
     my $path_sep = $Config::Config{path_sep} || ';';
     $option{inc} = [ split /\Q$path_sep\E/, join $path_sep,
@@ -78,9 +129,12 @@ sub new_with_options {
     return bless {
         option     => \%option,
         args       => $args,
+        callbacks  => \@cb,
         seen       => {},
         wallflower => Wallflower->new(
-            application => Plack::Util::load_psgi( $option{application} ),
+            application => ref $option{application}
+                ? $option{application}
+                : Plack::Util::load_psgi( $option{application} ),
             ( destination => $option{destination} )x!! $option{destination},
             ( index       => $option{index}       )x!! $option{index},
             ( url         => $option{url}         )x!! $option{url},
@@ -110,11 +164,10 @@ sub _process_args {
 }
 
 sub _process_queue {
-    my ( $self, @queue ) = @_;
-    my ( $quiet, $follow, $seen )
-        = @{ $self->{option} }{qw( quiet follow seen )};
-    my $wallflower = $self->{wallflower};
-    my $host_ok    = $self->_host_regexp;
+    my ( $self,       @queue ) = @_;
+    my ( $wallflower, $seen )  = @{$self}{qw( wallflower seen )};
+    my $follow  = $self->{option}{follow};
+    my $host_ok = $self->_host_regexp;
 
     # I'm just hanging on to my friend's purse
     local $ENV{PLACK_ENV} = $self->{option}{environment};
@@ -128,13 +181,12 @@ sub _process_queue {
 
         # get the response
         my $response = $wallflower->get($url);
-        my ( $status, $headers, $file ) = @$response;
 
-        # tell the world
-        printf "$status %s%s\n", $url->path, $file && " => $file [${\-s $file}]"
-            if !$quiet;
+        # run the callbacks
+        $_->( $url => $response ) for @{ $self->{callbacks} };
 
         # obtain links to resources
+        my ( $status, $headers, $file ) = @$response;
         if ( $status eq '200' && $follow ) {
             push @queue, links_from( $response => $url );
         }
@@ -160,9 +212,8 @@ sub _host_regexp {
 
 # ABSTRACT: Class performing the moves for the wallflower program
 
-
-
 __END__
+
 =pod
 
 =head1 NAME
@@ -171,7 +222,7 @@ App::Wallflower - Class performing the moves for the wallflower program
 
 =head1 VERSION
 
-version 1.005
+version 1.006
 
 =head1 SYNOPSIS
 
@@ -193,6 +244,13 @@ and return a object ready to be C<run()>.
 
 See L<wallflower> for the list of options and their usage.
 
+=head2 new( option => \%option, args => \@args )
+
+Create an object ready to be C<run()>.
+
+C<option> is a hashref of options as produced by L<Getopt::Long>, and
+C<args> is an array ref of optional arguments to be processed by C<run()>
+
 =head2 run( )
 
 Make L<wallflower> dance.
@@ -212,4 +270,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
